/**
Converts a JPEG image into a raw byte-by-byte RGBA array.

This library requires the osl::graphics2d library to link;
sadly, this library is currently too huge to publish...

Linux:
g++ -O -I.  -I../../build//../lib -Wall -c jpeg2raw.cpp
g++ -O -I.  -I../../build//../lib -Wall -DALWAYS_UPLOAD_RGBA=1 -I/usr/include -I/usr/local/include -I/usr/X11R6/include  -I. -I../../build//ext/osl/include -L../../build//../lib  -L/usr/lib -L/usr/local/lib -L/usr/X11R6/lib  -o jpeg2raw jpeg2raw.o -L../../build//../lib/ ../../build//ext/osl/lib/osl_graphics2d.a -lpng -lz ../../build//ext/osl/lib/osl_io.a -ljpeg ../../build//ext/osl/lib/osl_math.a ../../build//ext/osl/lib/osl.a

Windows:
link /out:jpeg2raw.exe jpeg2raw.obj  ../../build//ext/osl/lib.win32/osl_graphics2d.lib ../../build//ext/osl/lib.win32/libpng.lib ../../build//ext/osl/lib.win32/libz.lib ../../build//ext/osl/lib.win32/osl_io.lib ../../build//ext/osl/lib.win32/libjpeg.lib  ../../build//ext/osl/lib.win32/osl_math.lib ../../build//ext/osl/lib.win32/osl.lib 

Orion Sky Lawlor, olawlor@acm.org, 2007/02/06 (Public Domain)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <fstream>

/* For reading JPEG files */
#include "osl/raster.h"

int main(int argc,char **argv)
{
/* Read image */
	osl::graphics2d::RgbaRaster rast;
	const char *fName="volume.jpg";
	if (argc>1) fName=argv[1];
	rast.readNoThrow(fName);
	int w=rast.wid;
	int h=w;
	int rh=h;
	int d=rast.ht/rh;
	/* 3D textures must have power-of-two sides: */
	w=osl::roundUp2(w);
	h=osl::roundUp2(h);
	d=osl::roundUp2(d);

/* Put slices into 3D array */
	printf("Read %d x %d image; Allocating %d x %d x %d 3D texture\n",
		rast.wid,rast.ht, w,h,d);
	unsigned char *arr=new unsigned char[4*w*h*d];
	for (int z=0;z<d;z++)
	for (int y=0;y<h;y++)
	for (int x=0;x<w;x++) {
		int rx=x;
		int ry=z*rh+y;
		osl::graphics2d::RgbaPixel p(0u); /* black pixel */
		if (rx<rast.wid && (y<rh) && ry<rast.ht) 
		{ /* we're in-bounds on the source image: read source image color */
			p=rast.at(rx,ry);
		}
		int i=((z*h+y)*w+x)*4;
		arr[i+0]=(unsigned char)(p.r()); /* Red */
		arr[i+1]=(unsigned char)(p.g()); /* Green */
		arr[i+2]=(unsigned char)(p.b()); /* Blue */
		arr[i+3]=0; /* premultiplied alpha == zero means additive color */
	}
/* Write out volume to .raw and .siz files */
	printf("Copied to array; now writing to disk\n");
	FILE *f=fopen("volume.raw","wb");
	if (f) {
		fwrite(arr,4*w*h*d,1,f);
		fclose(f);
	} else {
		printf("Error creating volume.raw!\n");
	}
	std::ofstream hdr("volume.siz");
	hdr<<"V4 "<<w<<" "<<h<<" "<<d<<"\n";
	return 0;
}
